#include <WiFi.h>
#include <WebServer.h>
#include <EEPROM.h>
#include <NTPClient.h>
#include <WiFiUdp.h>

// Configuration WiFi
const char* ssid = "....";
const char* password = "....";

// Broches des vannes 
#define VANNE1 5
#define VANNE2 18

// Paramètres du cycle
volatile unsigned long dureeCycle = 5 * 60 * 1000; // 5 minutes par défaut
const unsigned long dureeMaxManuelle = 3600000; // 1 heure (3600s * 1000ms)

// Variables d'état
enum EtatSysteme {ARRET, EN_VANNE1, EN_VANNE2, MANU_VANNE1, MANU_VANNE2};
volatile EtatSysteme etat = ARRET;
bool vanne1Active = false;
bool vanne2Active = false;
unsigned long startTime = 0;
WebServer server(80);

// Variables pour le temps d'arrosage
unsigned long dureeArrosage1 = 0;
unsigned long dureeArrosage2 = 0;

// Variables pour l'horloge NTP
WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, "pool.ntp.org", 7200, 60000);

// Variables pour l'automatisation
bool automatiqueActif = false;
int triggerHeure = 8;
int triggerMinute = 0;
bool declenche = false;

// Structure pour la sauvegarde EEPROM
struct PersistentData {
  uint8_t duree;
  uint8_t triggerHeure;
  uint8_t triggerMinute;
  bool automatiqueActif;
};

// Page HTML - Version compacte
const char PAGE_HTML[] PROGMEM = R"rawliteral(
<!DOCTYPE html>
<html>
<head>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta charset="UTF-8">
  <title>Contrôle Arrosage</title>
  <style>
    * { box-sizing: border-box; margin: 0; padding: 0; }
    body { 
      font-family: Arial, sans-serif; 
      background-color: #f0f0f0; 
      padding: 10px;
      display: flex;
      justify-content: center;
    }
    .content-wrapper {
      width: 100%;
      max-width: 400px;
      background: white;
      border-radius: 12px;
      box-shadow: 0 3px 8px rgba(0,0,0,0.1);
      padding: 15px;
    }
    h1 { 
      color: #2c3e50; 
      margin-bottom: 12px; 
      font-size: 1.4rem;
      text-align: center;
    }
    .ip-info {
      font-size: 0.9rem;
      color: #2c3e50;
      margin-bottom: 10px;
      text-align: center;
    }
    .clock-container {
      background: #2c3e50;
      color: white;
      padding: 8px 15px;
      border-radius: 6px;
      margin-bottom: 12px;
      font-size: 1.1rem;
      text-align: center;
    }
    .sync-info {
      font-size: 0.7rem;
      color: #2c3e50;
      margin-bottom: 15px;
      text-align: center;
    }
    .status-container {
      display: flex;
      justify-content: space-around;
      margin-bottom: 15px;
    }
    .status-item {
      display: flex;
      flex-direction: column;
      align-items: center;
      min-width: 80px;
    }
    .status-label {
      font-size: 0.9rem;
      margin-bottom: 5px;
      color: #2c3e50;
    }
    .led {
      width: 22px;
      height: 22px;
      border-radius: 50%;
      background: #ccc;
      margin-bottom: 5px;
    }
    .led-on {
      background: #4CAF50;
      box-shadow: 0 0 6px #4CAF50;
    }
    .temps-restant {
      font-size: 0.8rem;
      min-height: 16px;
    }
    .section-title {
      font-size: 1.0rem;
      color: #2c3e50;
      margin: 12px 0 8px;
      text-align: center;
    }
    .btn-container {
      display: flex;
      flex-wrap: wrap;
      justify-content: center;
      gap: 8px;
      margin-bottom: 15px;
    }
    .btn {
      min-width: 120px;
      padding: 8px 6px;
      font-size: 0.85rem;
      border: none;
      border-radius: 6px;
      cursor: pointer;
      box-shadow: 0 2px 4px rgba(0,0,0,0.1);
    }
    .btn-on { background: #4CAF50; color: white; }
    .btn-off { background: #f44336; color: white; }
    .btn-manu1 { background: #2196F3; color: white; }
    .btn-manu2 { background: #FF9800; color: white; }
    .config-container {
      background: #f9f9f9;
      padding: 10px;
      border-radius: 8px;
      margin-bottom: 12px;
    }
    .config-title {
      font-size: 0.9rem;
      margin-bottom: 8px;
      text-align: center;
    }
    .config-input {
      display: flex;
      justify-content: center;
      align-items: center;
      gap: 6px;
      margin-bottom: 8px;
    }
    input {
      padding: 8px;
      font-size: 0.9rem;
      width: 60px;
      text-align: center;
      border: 1px solid #ddd;
      border-radius: 4px;
    }
    .set-btn {
      padding: 8px 10px;
      background: #2196F3;
      color: white;
      border: none;
      border-radius: 4px;
      cursor: pointer;
      font-size: 0.85rem;
    }
    .auto-row {
      display: flex;
      justify-content: center;
      gap: 6px;
      margin-bottom: 8px;
    }
    .auto-status {
      font-size: 0.9rem;
      text-align: center;
      min-height: 18px;
      margin-top: 6px;
    }
  </style>
</head>
<body>
  <div class="content-wrapper">
    <h1>Arrosage Piscine</h1>
    
    <div class="ip-info" id="ipAddress">IP: ...</div>
    
    <div class="clock-container">
      <div id="currentTime">00:00:00</div>
    </div>
    <div class="sync-info">Heure synchronisée</div>
    
    <div class="status-container">
      <div class="status-item">
        <div class="status-label">VANNE 1</div>
        <div id="led1" class="led"></div>
        <div id="temps1" class="temps-restant">0s</div>
      </div>
      <div class="status-item">
        <div class="status-label">VANNE 2</div>
        <div id="led2" class="led"></div>
        <div id="temps2" class="temps-restant">0s</div>
      </div>
    </div>
    
    <div class="section-title">Mode Manuel</div>
    <div class="btn-container">
      <button class="btn btn-manu1" onclick="controlVanne1()">VANNE 1</button>
      <button class="btn btn-manu2" onclick="controlVanne2()">VANNE 2</button>
    </div>
    
    <div class="config-container">
      <div class="config-title">Durée du cycle (minutes)</div>
      <div class="config-input">
        <input type="number" id="duree" min="1" max="60" value="5">
        <button class="set-btn" onclick="setDuree()">OK</button>
      </div>
    </div>
    
    <div class="config-container">
      <div class="config-title">Cycle Automatique</div>
      <div class="auto-row">
        <label for="autoActif">Activer:</label>
        <input type="checkbox" id="autoActif" onchange="updateAutoStatus()">
      </div>
      <div class="config-title">Heure déclenchement</div>
      <div class="config-input">
        <input type="number" id="autoHeure" min="0" max="23" placeholder="H">
        <input type="number" id="autoMinute" min="0" max="59" placeholder="M">
        <button class="set-btn" onclick="setAuto()">OK</button>
      </div>
      <div id="autoInfo" class="auto-status"></div>
    </div>
    
    <div class="btn-container">
      <button class="btn btn-on" onclick="control('on')">DÉMARRER</button>
      <button class="btn btn-off" onclick="control('off')">ARRÊTER</button>
    </div>
  </div>

  <script>
    function updateStatus(data) {
      document.getElementById('led1').className = data.vanne1 ? 'led led-on' : 'led';
      document.getElementById('led2').className = data.vanne2 ? 'led led-on' : 'led';
      document.getElementById('temps1').textContent = formatTemps(data.dureeArrosage1);
      document.getElementById('temps2').textContent = formatTemps(data.dureeArrosage2);
      document.getElementById('currentTime').textContent = 
        data.heure.toString().padStart(2, '0') + ':' +
        data.minute.toString().padStart(2, '0') + ':' +
        data.seconde.toString().padStart(2, '0');
      document.getElementById('ipAddress').textContent = 'IP: ' + data.ip;
    }
    
    function formatTemps(secondes) {
      if (secondes <= 0) return "0s";
      const min = Math.floor(secondes / 60);
      const sec = secondes % 60;
      if (min > 0) return `${min}min ${sec}s`;
      return `${sec}s`;
    }
    
    function control(cmd) {
      fetch('/' + cmd);
    }
    
    function controlVanne1() {
      fetch('/on1');
    }
    
    function controlVanne2() {
      fetch('/on2');
    }
    
    function setDuree() {
      const duree = document.getElementById('duree').value;
      fetch('/setduree?d=' + duree);
    }
    
    function setAuto() {
      const actif = document.getElementById('autoActif').checked ? '1' : '0';
      const heure = document.getElementById('autoHeure').value;
      const minute = document.getElementById('autoMinute').value;
      fetch('/setauto?a=' + actif + '&h=' + heure + '&m=' + minute)
        .then(() => updateAutoStatus());
    }
    
    function updateAutoStatus() {
      const actif = document.getElementById('autoActif').checked;
      const heure = document.getElementById('autoHeure').value.padStart(2, '0') || '00';
      const minute = document.getElementById('autoMinute').value.padStart(2, '0') || '00';
      document.getElementById('autoInfo').textContent = 
        actif ? `Activé à ${heure}:${minute}` : 'Désactivé';
    }
    
    fetch('/init')
      .then(response => response.json())
      .then(data => {
        document.getElementById('duree').value = data.duree;
        document.getElementById('autoActif').checked = data.autoActif;
        document.getElementById('autoHeure').value = data.autoHeure;
        document.getElementById('autoMinute').value = data.autoMinute;
        document.getElementById('ipAddress').textContent = 'IP: ' + data.ip;
        updateAutoStatus();
      });
    
    setInterval(() => {
      fetch('/status')
        .then(response => response.json())
        .then(data => updateStatus(data));
    }, 1000);
  </script>
</body>
</html>
)rawliteral";

// Variables pour le temps d'arrosage manuel
unsigned long startManuel1 = 0;
unsigned long startManuel2 = 0;

// Variables pour la gestion WiFi
unsigned long dernierControleWifi = 0;
const unsigned long intervalleControleWifi = 10000;
byte tentativesReconnexion = 0;
const byte maxTentatives = 5;

void setup() {
  Serial.begin(115200);
  pinMode(VANNE1, OUTPUT);
  pinMode(VANNE2, OUTPUT);
  digitalWrite(VANNE1, LOW);
  digitalWrite(VANNE2, LOW);
  
  // Initialisation EEPROM
  EEPROM.begin(sizeof(PersistentData));
  chargerConfig();
  
  // Configuration WiFi
  WiFi.mode(WIFI_STA);
  WiFi.setAutoReconnect(true);
  WiFi.persistent(true);
  
  // Connexion WiFi
  Serial.print("Connexion WiFi ");
  WiFi.begin(ssid, password);
  
  // Timeout pour la connexion WiFi
  unsigned long startAttemptTime = millis();
  while (WiFi.status() != WL_CONNECTED && millis() - startAttemptTime < 10000) {
    delay(500);
    Serial.print(".");
  }
  
  if (WiFi.status() != WL_CONNECTED) {
    Serial.println("\nÉchec de connexion WiFi!");
  } else {
    Serial.println("\nConnecté! IP: " + WiFi.localIP().toString());
  }

  // Initialisation NTP
  timeClient.begin();
  timeClient.forceUpdate();
  Serial.println("Heure NTP: " + timeClient.getFormattedTime());

  // Configuration serveur web
  server.on("/", []() { server.send(200, "text/html", PAGE_HTML); });
  server.on("/on", demarrerCycle);
  server.on("/on1", demarrerVanne1Manuel);
  server.on("/on2", demarrerVanne2Manuel);
  server.on("/off", arreterCycle);
  server.on("/setduree", handleSetDuree);
  server.on("/status", handleStatus);
  server.on("/setauto", handleSetAuto);
  server.on("/init", handleInit);
  
  server.begin();
  Serial.println("Serveur web démarré");
}

void loop() {
  gererConnexionWifi();
  server.handleClient();
  timeClient.update();
  verifierMinuit();
  reinitialiserDeclenchementSiNouvelleHeure();
  gestionCycle();
  gestionAuto();
  gestionManuel();
  delay(10);
}

void gererConnexionWifi() {
  if (millis() - dernierControleWifi > intervalleControleWifi) {
    if (WiFi.status() != WL_CONNECTED) {
      Serial.println("WiFi déconnecté, tentative de reconnexion...");
      tentativesReconnexion++;
      
      WiFi.disconnect();
      WiFi.reconnect();
      
      if (tentativesReconnexion > maxTentatives) {
        Serial.println("Redémarrage dû à l'échec de reconnexion WiFi");
        ESP.restart();
      }
    } else {
      tentativesReconnexion = 0;
    }
    dernierControleWifi = millis();
  }
}

void gestionManuel() {
  if (etat == MANU_VANNE1) {
    dureeArrosage1 = (millis() - startManuel1) / 1000;
    if (millis() - startManuel1 >= dureeMaxManuelle) {
      arreterVannes();
      Serial.println("Arrêt sécurité vanne 1");
    }
  }
  
  if (etat == MANU_VANNE2) {
    dureeArrosage2 = (millis() - startManuel2) / 1000;
    if (millis() - startManuel2 >= dureeMaxManuelle) {
      arreterVannes();
      Serial.println("Arrêt sécurité vanne 2");
    }
  }
}

void verifierMinuit() {
  static bool aMinuit = false;
  int h = timeClient.getHours();
  int m = timeClient.getMinutes();
  int s = timeClient.getSeconds();
  
  if (h == 0 && m == 0 && s == 0 && !aMinuit) {
    declenche = false;
    aMinuit = true;
  } else if (h != 0 || m != 0 || s != 0) {
    aMinuit = false;
  }
}

void reinitialiserDeclenchementSiNouvelleHeure() {
  int h = timeClient.getHours();
  int m = timeClient.getMinutes();
  
  if (h > triggerHeure || (h == triggerHeure && m > triggerMinute)) {
    if (declenche) declenche = false;
  }
}

void gestionAuto() {
  if (!automatiqueActif || etat != ARRET || declenche) return;

  int h = timeClient.getHours();
  int m = timeClient.getMinutes();
  
  if (h == triggerHeure && m == triggerMinute) {
    demarrerCycleInterne();
    declenche = true;
    Serial.println("Cycle automatique démarré");
  }
}

void chargerConfig() {
  PersistentData data;
  EEPROM.get(0, data);
  
  if (data.duree >= 1 && data.duree <= 60 && 
      data.triggerHeure < 24 && data.triggerMinute < 60) {
    
    dureeCycle = data.duree * 60 * 1000;
    automatiqueActif = data.automatiqueActif;
    triggerHeure = data.triggerHeure;
    triggerMinute = data.triggerMinute;
  } else {
    dureeCycle = 5 * 60 * 1000;
    automatiqueActif = false;
    triggerHeure = 8;
    triggerMinute = 0;
    sauvegarderConfig();
  }
}

void sauvegarderConfig() {
  PersistentData data = {
    .duree = (uint8_t)(dureeCycle / (60 * 1000)),
    .triggerHeure = (uint8_t)triggerHeure,
    .triggerMinute = (uint8_t)triggerMinute,
    .automatiqueActif = automatiqueActif
  };
  
  EEPROM.put(0, data);
  EEPROM.commit();
}

void demarrerCycleInterne() {
  if (etat == ARRET) {
    etat = EN_VANNE1;
    vanne1Active = true;
    digitalWrite(VANNE1, HIGH);
    startTime = millis();
    dureeArrosage1 = 0;
    dureeArrosage2 = 0;
  }
}

void demarrerVanne1Manuel() {
  arreterVannes();
  etat = MANU_VANNE1;
  vanne1Active = true;
  digitalWrite(VANNE1, HIGH);
  startManuel1 = millis();
  dureeArrosage1 = 0;
  dureeArrosage2 = 0;
  server.send(200, "text/plain", "Vanne 1 ON");
}

void demarrerVanne2Manuel() {
  arreterVannes();
  etat = MANU_VANNE2;
  vanne2Active = true;
  digitalWrite(VANNE2, HIGH);
  startManuel2 = millis();
  dureeArrosage1 = 0;
  dureeArrosage2 = 0;
  server.send(200, "text/plain", "Vanne 2 ON");
}

void demarrerCycle() {
  demarrerCycleInterne();
  server.send(200, "text/plain", "Cycle démarré");
}

void arreterCycle() {
  arreterVannes();
  etat = ARRET;
  vanne1Active = false;
  vanne2Active = false;
  dureeArrosage1 = 0;
  dureeArrosage2 = 0;
  server.send(200, "text/plain", "Arrêt");
}

void arreterVannes() {
  digitalWrite(VANNE1, LOW);
  digitalWrite(VANNE2, LOW);
}

void gestionCycle() {
  if (etat == ARRET || etat == MANU_VANNE1 || etat == MANU_VANNE2) return;

  unsigned long ecoule = millis() - startTime;

  switch (etat) {
    case EN_VANNE1:
      if (ecoule >= dureeCycle) {
        digitalWrite(VANNE1, LOW);
        digitalWrite(VANNE2, HIGH);
        etat = EN_VANNE2;
        vanne1Active = false;
        vanne2Active = true;
        startTime = millis();
      }
      break;
      
    case EN_VANNE2:
      if (ecoule >= dureeCycle) {
        arreterVannes();
        etat = ARRET;
        vanne1Active = false;
        vanne2Active = false;
      }
      break;
  }
}

void handleSetDuree() {
  if (server.hasArg("d")) {
    int d = server.arg("d").toInt();
    if (d >= 1 && d <= 60) {
      dureeCycle = d * 60 * 1000;
      sauvegarderConfig();
      server.send(200, "text/plain", "Durée mise à jour");
      return;
    }
  }
  server.send(400, "text/plain", "Valeur invalide");
}

void handleSetAuto() {
  if (server.hasArg("a") && server.hasArg("h") && server.hasArg("m")) {
    automatiqueActif = (server.arg("a") == "1");
    triggerHeure = server.arg("h").toInt();
    triggerMinute = server.arg("m").toInt();
    
    triggerHeure = (triggerHeure < 0) ? 0 : (triggerHeure > 23) ? 23 : triggerHeure;
    triggerMinute = (triggerMinute < 0) ? 0 : (triggerMinute > 59) ? 59 : triggerMinute;
    
    declenche = false;
    sauvegarderConfig();
    server.send(200, "text/plain", "Auto config mise à jour");
  } else {
    server.send(400, "text/plain", "Paramètres manquants");
  }
}

void handleInit() {
  String json = 
    "{\"duree\":" + String(dureeCycle / 60000) + 
    ",\"autoActif\":" + String(automatiqueActif ? "true" : "false") + 
    ",\"autoHeure\":" + String(triggerHeure) + 
    ",\"autoMinute\":" + String(triggerMinute) + 
    ",\"ip\":\"" + WiFi.localIP().toString() + "\"}";
    
  server.send(200, "application/json", json);
}

void handleStatus() {
  if (etat == EN_VANNE1) {
    dureeArrosage1 = (millis() - startTime) / 1000;
  } else if (etat == EN_VANNE2) {
    dureeArrosage2 = (millis() - startTime) / 1000;
  }
  
  String json = 
    "{\"vanne1\":" + String(vanne1Active ? "true" : "false") + 
    ",\"vanne2\":" + String(vanne2Active ? "true" : "false") + 
    ",\"dureeArrosage1\":" + String(dureeArrosage1) + 
    ",\"dureeArrosage2\":" + String(dureeArrosage2) + 
    ",\"heure\":" + String(timeClient.getHours()) + 
    ",\"minute\":" + String(timeClient.getMinutes()) + 
    ",\"seconde\":" + String(timeClient.getSeconds()) + 
    ",\"ip\":\"" + WiFi.localIP().toString() + "\"}";
    
  server.send(200, "application/json", json);
}